#!/bin/bash
# **********************
# Name 		: Source Mage GNU Linux - Server - FireWall 
# Author	: V. Vydra
# **********************

# *** Environment vars
VERSION=0.0.2
IPT=/usr/sbin/iptables
EXTERN="eth0"
LOOPBACK="lo"
LOOP=127.0.0.1
PRIVATE=10.0.0.0/24

start_fw()
{
    ##
    # *** Set kernel stuff
    ##
    
    # enable the kernel's spoof protection
    echo 1 > /proc/sys/net/ipv4/conf/all/rp_filter
    
    # This enables ip forwarding, and thus by extension, NAT
    # Turn this on if you're going to be doing NAT or Masquerading
    #echo 1 > /proc/sys/net/ipv4/ip_forward

    ##
    # *** Load modules
    ##
    
    modprobe ip_tables
    # to allow NATed ftpk
    modprobe ip_conntrack_ftp

    # *** Setup our iptables policies
    $IPT -P INPUT ACCEPT
    #DROP
    #REJECT --reject-with icmp-port-unreachable
    $IPT -P OUTPUT ACCEPT
    $IPT -P FORWARD ACCEPT

    ##
    # *** Create Chains
    ##
    
    # ** DROP all packets comming from reserved private IP addresses.
    $IPT -N SRC_EGRESS
    $IPT -F SRC_EGRESS
    # Class A Reserved
    $IPT -A SRC_EGRESS -s 10.0.0.0/8 -j DROP
    # Class B Reserved
    $IPT -A SRC_EGRESS -s 172.16.0.0/12 -j DROP
    # Class C Reserved
    $IPT -A SRC_EGRESS -s 192.168.0.0/16 -j DROP
    # Class D Reserved
    $IPT -A SRC_EGRESS -s 224.0.0.0/4 -j DROP
    # Class E Reserved
    $IPT -A SRC_EGRESS -s 240.0.0.0/5 -j DROP
    
    # ** DROP all packets destinated to reserved private IP addresses.
    $IPT -N DES_EGRESS
    $IPT -F DES_EGRESS
    # Class A Reserved
    $IPT -A DES_EGRESS -d 10.0.0.0/8 -j DROP
    # Class B Reserved
    $IPT -A DES_EGRESS -d 172.16.0.0/12 -j DROP
    # Class C Reserved
    $IPT -A DES_EGRESS -d 192.168.0.0/16 -j DROP
    # Class D Reserved
    $IPT -A DES_EGRESS -d 224.0.0.0/4 -j DROP
    # Class E Reserved
    $IPT -A DES_EGRESS -d 240.0.0.0/5 -j DROP
    
    
    ##
    # **** INPUT    
    ##
    
    # ** filter BAD net
    $IPT -A INPUT -i eth0 -j SRC_EGRESS
    $IPT -A INPUT -i eth0 -j DES_EGRESS

    
    # ** Loopback
    
    
    # Prevent external packets from using loopback addr
    $IPT -A INPUT -i eth0 -s $LOOP -j DROP
    $IPT -A FORWARD -i eth0 -s $LOOP -j DROP
    $IPT -A INPUT -i eth0 -d $LOOP -j DROP
    $IPT -A FORWARD -i eth0 -d $LOOP -j DROP
    
    # Allow local loopback
    $IPT -A INPUT -s $LOOP -j ACCEPT
    $IPT -A INPUT -d $LOOP -j ACCEPT
    # accept all loopback to loopback traffick
    #$IPT -A INPUT -i $LOOPBACK -j ACCEPT
    
    ##    
    # *** Services
    ##
    
    # ** Allow incoming pings (can be disabled)
    $IPT -A INPUT -p icmp --icmp-type echo-request -j ACCEPT
    
    # ** Allow http
    $IPT -A INPUT -p tcp --dport http -j ACCEPT
    $IPT -A INPUT -p tcp --dport 443 -j ACCEPT
     
    # ** Allow ssh
    $IPT -A INPUT -p tcp --dport ssh -j ACCEPT

    # ** Allow ftp
    $IPT -A INPUT -p tcp --dport ftp -j ACCEPT

    # ** Allow dcgui
    #$IPT -A INPUT -p tcp --dport 2412 -j ACCEPT
    #$IPT -A INPUT -p udp --dport 2412 -j ACCEPT
    
    
    # ** Allow incoming smtp connects (mailservers)
    $IPT -A INPUT -p tcp --dport smtp -j ACCEPT

    # ** Allow pgsql
    #$IPT -A INPUT -p tcp -s 127.0.0.1 --dport 5432 -j ACCEPT 

    # ** Allow Mail reading - secure way
    $IPT -A INPUT -p tcp --dport imaps -j ACCEPT
    $IPT -A INPUT -p tcp --dport pop3s -j ACCEPT
    #$IPT -A INPUT -p tcp --dport imap -j ACCEPT
    #$IPT -A INPUT -p tcp --dport pop3 -j ACCEPT
    
    # ** Allow incoming DNS queries (DNS servers)
    $IPT -A INPUT -p udp --dport domain -j ACCEPT
    $IPT -A INPUT -p tcp --dport domain -j ACCEPT
    
    # ** Allow incoming NTP queries (NTP timeservers)
    #$IPT -A INPUT -p udp --dport ntp -j ACCEPT

    # ** Block outgoing NetBios
    $IPT -A FORWARD -p tcp --sport 137:139 -o eth0 -j DROP
    $IPT -A FORWARD -p udp --sport 137:139 -o eth0 -j DROP
    $IPT -A OUTPUT -p tcp --sport 137:139 -o eth0 -j DROP
    $IPT -A OUTPUT -p udp --sport 137:139 -o eth0 -j DROP
    
    # ** Allow Unreal Tournament :))
    # UDP Ports 2000-2009 for Pinging Internet Servers
    $IPT -A INPUT -p udp --dport 2000:2009 -j ACCEPT
    # Server Query Receive UDP Port (7777 by default)
    $IPT -A INPUT -p udp --dport 7777 -j ACCEPT
    # Master Server "Heartbeat" UDP Port (Always Server Query Receive Port + 1, usually 7778)
    $IPT -A INPUT -p udp --dport 7778 -j ACCEPT
    # Master Server "Heartbeat" UDP Port (Always Server Query Receive Port + 1, usually 7778)
    $IPT -A INPUT -p udp --dport 7779 -j ACCEPT
    # Master Server "Heartbeat" UDP Port (Always Server Query Receive Port + 1, usually 7778)
    $IPT -A INPUT -p udp --dport 7780 -j ACCEPT
    # Master Server "Heartbeat" UDP Port (Always Server Query Receive Port + 1, usually 7778)
    $IPT -A INPUT -p udp --dport 7781 -j ACCEPT
    # LAN Beacon UDP Port (9777 by default)
    $IPT -A INPUT -p udp --dport 8777 -j ACCEPT
    
    # ** Allow incoming OpenVPN packets
    
    ##$IPT -A INPUT -p udp --dport 5000 -j vvPEARS
    ##ACCEPT
    #$IPT -A INPUT -p udp  -s $OPENVP_HOST1 --dport 5000 -j ACCEPT
    #$IPT -A INPUT -p udp  -s $OPENVP_HOST2 --dport 5001 -j ACCEPT    
    #$IPT -A INPUT -p udp  -s $OPENVP_HOST3 --dport 5002 -j ACCEPT    
    
    # Allow packets from TUN/TAP devices.
#    $IPT -A INPUT -i tun+ -j ACCEPT
#    $IPT -A FORWARD -i tun+ -j ACCEPT
#    $IPT -A INPUT -i tap+ -j ACCEPT
#    $IPT -A FORWARD -i tap+ -j ACCEPT
    
    ## Allow packets from private subnets
    ##$IPT -A INPUT -i eth1 -j ACCEPT
    ##$IPT -A FORWARD -i eth1 -j ACCEPT
    
    # Keep state of connections from local machine and private subnets
    $IPT -A OUTPUT -m state --state NEW -o eth0 -j ACCEPT
    $IPT -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
    $IPT -A FORWARD -m state --state NEW -o eth0 -j ACCEPT
    $IPT -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT
    
    # Masquerade local subnet
    #$IPT -t nat -A POSTROUTING -s $PRIVATE -o eth0 -j MASQUERADE
    
    #modprobe ipt_MASQUERADE
    #iptables -t nat -A POSTROUTING -o eth0 -j SNAT --to $MASKARADE_TO_IP
    #echo 1 > /proc/sys/net/ipv4/ip_forward

    # on the private eth1 - allow ALL but only local subnet
    #$IPT -A INPUT -i eth1 -s 10.0.0.0/8 -j ACCEPT
    #$IPT -A INPUT -i eth1 -j REJECT --reject-with icmp-port-unreachable
    
    # REJECT ALL
    $IPT -A INPUT -i eth0 -j REJECT --reject-with icmp-port-unreachable
}

stop_fw()
{
    # clear out INPUT
    #for ch in INPUT ; do
    #    $IPT -F $ch
    #    $IPT -Z $ch
    #done	
    
    # clear out vvPEARS
    #for ch in vvPEARS ; do
    #    $IPT -F $ch
    #    $IPT -Z $ch
    #done	
    
    $IPT -F
    $IPT -t nat -F
    $IPT -t mangle -F
    
    $IPT -X vvPEARS
    $IPT -X DES_EGRESS
    $IPT -X SRC_EGRESS
}


# *** main case
case "$1" in
        start)
	    echo -n "Starting Source Mage Server iptables firewall ($VERSION) ... "
	    start_fw
	    echo "done"
	    ;;
	stop)
	    echo -n "Clearing  Source Mage Server iptables firewall ... "
            stop_fw
            echo "done"
            ;;
        restart|reload|force-reload)
            $0 stop
            $0 start
            ;;
	install)
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc0.d/K69sm-firewall
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc1.d/K69sm-firewall
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc2.d/K69sm-firewall
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc6.d/K69sm-firewall
						    
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc3.d/S31sm-firewall
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc4.d/S31sm-firewall
	    ln  -sf  ../init.d/sm-firewall.sh  /etc/rc5.d/S31sm-firewall
	    ;;
        *)
            echo "Usage: $0 {start|stop|restart|reload|force-reload}"
            exit 1
            ;;
esac

exit 0
